#pragma once
/*
This .h applies to Gem.cpp and .cpp files which do not have separate .h files.

VwEnergy - Energy Landscape 
VwData - View data files

*/
/* 
TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO 
Use float or doubles ? 
*/

#include "resource.h"

#include "MathUtils.h"

class CDataFileInterface;
#include "Data.h"
#include "QF.h"
#include "DataFile.h"

#include "Model.h"
#include "Movie.h"

#include "Graphs.h"

#include "GemConfig.h"

HWND GetClientWin( );
HINSTANCE GetInstance( );
const char * GetChildClassName();
void DataMenuOn();
void DataMenuOff();
void ModelMenuOn();
void ModelMenuOff();
HWND OpenDlgWindow( const char * pchWindowName, int iID, DLGPROC lpDialogFunc );

HWND OpenDataView( const char * pchWindowName );
HWND OpenModelView( const char * pchWindowName );
HWND OpenMovieView( const char * pchWindowName );
HWND OpenGraphsView( const char * pchWindowName );
HWND OpenHistogramView( const char * pchWindowName );

void UpdateModelMRU( const char * pchName );
void UpdateDataMRU( const char * pchName );
void UpdateMovieMRU( const char * pchName );

//-- window id's and get data usage
void GetFileNameList( int iID, CStringArray * pac );

int FillModelCombo( CDlgCombo * pcmb, CStringArray * pac=NULL, CPtrArray * pahwnd=NULL );

CString SaveWorkspace( ); 
void LoadWorkspace( const char * pchKey );
	
//-- User messages to main window 
const int WM_ADD_HISTOGRAM = WM_USER+1;

//-- Menu extras
const int IDM_MRUWORKSPACE = WM_USER+200;
const int IDM_MRUMODEL = WM_USER+220;
const int IDM_MRUDATA = WM_USER+240;
const int IDM_MRUMOVIE = WM_USER+260;

void MenuItemCheck( int iID, bool bCheck );

//-- Thread communication
void ThreadErrorMessage( const char * pchMsg, void * pInfo=NULL );
void ThreadProgress( int iPct );

//----------------------- DlgSingleEntry
bool GetSingleEntry( HWND hWndOwner, const char * pchTitle, const char * pchText, CString * pc );

//----------------------- DlgListItem
bool EditListItem( HWND hWndOwner, CString * pcLabel, CString * pcComment );

//----------------------- DlgCopyImage
class CCopyImageProperties { 
	public : 
		int m_iWidth;
		int m_iHeight;
		bool m_bVector;
		void Set( int iWidth, int iHeight, bool bVector );
	};
bool GetCopyImageProperties( HWND hWndOwner, CCopyImageProperties * pProp );

//----------------------- DlgGraphProperties
bool GetGraphProperties( HWND hWndOwner, CGraph * pGraph );	

//----------------------- DlgModelGenerator.cpp
bool OpenModelGenerator( HWND hWndOwner );

//----------------------- VwEnergy.cpp
const int WM_MODEL_OPEN = WM_USER+1;
//BOOL CALLBACK EnergyDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK EnergyWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

//----------------------- VwData.cpp
const UINT WM_DATA_LOADFILE = WM_USER + 1;	// Open a data file
const UINT WM_DATA_LOADED = WM_USER + 2;	// notify data view a data segment has been loaded 
const UINT WM_DATA_CHANGED = WM_USER + 3;	// notify data view a data segment has been modified.  wParam!=0 : activate window
const UINT WM_DATA_ADDSEGMENT = WM_USER+4;  // add a segment of doubles, 1 channel.  wParam=pointcount; lParam=buffer to copy.
const UINT WM_DATA_CREATENEW = WM_USER+5;	// intialize a new empty file  
const UINT WM_DATA_CREATESIM = WM_USER+6;	// intialize a new empty file  wParam = COptionsSimulate ptr.
const UINT WM_DATA_ADDIDLSEG = WM_USER+7;	// add a segment of doubles, 1 channel with idealization.  wParam=pointcount; lParam=buffer to copy.
const UINT WM_DATA_CHANNELINFO = WM_USER+8;	// Set channel info wParam=channel index lParam="Units"
BOOL CALLBACK DataviewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);

//----------------------- VwDataConfig
bool OpenDataViewConfig( HWND hDlg, CDataConfigure * pConfig );
BOOL CALLBACK DataConfigDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam );
bool OpenDataDefaultConfig( HWND hDlg); 

//----------------------- VwModel.cpp
const UINT WM_MODEL_LOADFILE = WM_USER + 1;	// Open a model file
const UINT WM_MODEL_NEWFILE = WM_USER + 2;	// Create a new model file
const UINT ID_ENERGYMODEL_VIEW = WM_USER + 3;	// message used for viewing current model as energy model
const UINT WM_MODEL_SETMODEL = WM_USER + 4;	// copy passed model into view model  lParam = CModel *
const UINT WM_MODEL_CHANGED = WM_USER + 5;	// Some other function has modified a model and it should be updated
BOOL CALLBACK ModelViewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);

//----------------------- VwIdealize
void OpenVwIdealize( HWND hDlg, CData * pData, bool bList, const char * pchList,
						bool bSelect, int iSeg0, int iOffset0, int iSeg1, int iOffset1 );

//----------------------- VwTaskSimulate
void OpenVwSimulate( HWND hDlg, CModel * pModel ) ;

//----------------------- VwTaskMil
void OpenVwMil( HWND hDlg, CModel * pModel ) ;

//----------------------- VwTaskAmpHistogram
void OpenVwAmpHistogram( HWND hDlgMain, CData * pData, bool bList, const char * pchList, 
							bool bSelect, int iSeg0, int iOffset0, int iSeg1, int iOffset1 );

//----------------------- VwExport
void OpenVwExport( HWND hDlgVwData, CData * pData, bool bList, bool bSelect );

//----------------------- VwMovie.cpp
const UINT WM_MOVIE_LOADFILE = WM_USER + 1;	// Open a Movie file
const UINT WM_MOVIE_NEWFILE = WM_USER + 2;	// Create a new Movie file
BOOL CALLBACK MovieViewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
		
//----------------------- VwGraphs
void AddGraph( CGraph * pGraph );		
BOOL CALLBACK GraphsViewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
		
//----------------------- VwDataHistogram		
const int IDLHISTBINCOUNT = 600;
const int WM_IDLHIST_OPEN = WM_USER+1;
LRESULT CALLBACK HistogramWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

//----------------------- VwModelMerge
class COptionsModelMerge {
	public :
		int m_iModel1;
		CString m_cModel1;
		int m_iChannels1;
		bool m_bModel2;
		int m_iModel2;
		CString m_cModel2;
		int m_iChannels2;

		COptionsModelMerge();
		void ToBuffer( BYTE * buf );
		void FromBuffer( BYTE * buf );
		bool RegistryLoad();
		bool RegistrySave();
	};
void OpenVwModelMerge( HWND hDlg );
BOOL CALLBACK VwModelMergeDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
